package com.mcwl.resource.util;


import com.mcwl.common.utils.StringUtils;
import com.mcwl.resource.handler.IMessageHandler;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import static com.mcwl.common.utils.Threads.sleep;

@Slf4j
@Getter
@Configuration
@ConfigurationProperties(prefix = "mqtt")
public class MqttTemplate implements MqttCallbackExtended, DisposableBean {

    @Value("${mqtt.broker-url}")
    private String brokerUrl;

    @Value("${mqtt.client-id-prefix}")
    private String clientIdPrefix;

    @Value("${mqtt.connection-timeout}")
    private int connectionTimeout;

    @Value("${mqtt.keep-alive-interval}")
    private int keepAliveInterval;

    @Value("${mqtt.max-reconnect-attempts}")
    private int maxReconnectAttempts;

    @Value("${mqtt.clean-session}")
    private boolean cleanSession;

    private String clientId;

    private MqttAsyncClient client;
    private final Map<String, List<IMessageHandler>> topicHandlers = new ConcurrentHashMap<>();
    private final ScheduledExecutorService reconnectExecutor = Executors.newSingleThreadScheduledExecutor();
    private volatile boolean isConnecting;
    private final ApplicationContext context;

    public MqttTemplate(ApplicationContext context) {
        this.context = context;
    }


    @PostConstruct
    public void init() throws MqttException {
        if (StringUtils.isBlank(clientId)) {
            clientId = clientIdPrefix + UUID.randomUUID().toString().substring(0, 8);
        }

        client = new MqttAsyncClient(brokerUrl, clientId, new MemoryPersistence());
        client.setCallback(this);
        connect();
//        autoRegisterHandlers();
    }

    @Override
    public void destroy() throws Exception {
        if (client != null && client.isConnected()) {
            client.disconnect();
        }
        reconnectExecutor.shutdown();
    }

    /**
     * 连接管理
     *
     * @throws MqttException 连接失败
     */
    private void connect() throws MqttException {
        MqttConnectOptions options = new MqttConnectOptions();
        options.setCleanSession(cleanSession);
        options.setConnectionTimeout(connectionTimeout);
        options.setKeepAliveInterval(keepAliveInterval);
        options.setAutomaticReconnect(false); // 手动控制重连

        client.connect(options, null, new IMqttActionListener() {
            @Override
            public void onSuccess(IMqttToken asyncActionToken) {
                log.info("MQTT连接成功");
                resubscribeAllTopics();
                autoRegisterHandlers();
            }

            @Override
            public void onFailure(IMqttToken asyncActionToken, Throwable exception) {
                log.error("初始连接失败", exception);
                scheduleReconnect(1);
            }
        });
    }

    /**
     * 重连管理
     * @param attempt 重连次数
     */
    private void scheduleReconnect(int attempt) {
        if (attempt > maxReconnectAttempts) {
            log.error("达到最大重连次数: {}", maxReconnectAttempts);
            return;
        }

        long delay = (long) Math.min(1000 * Math.pow(2, attempt), 30000);
        reconnectExecutor.schedule(() -> {
            try {
                log.info("尝试第{}次重连", attempt);
                if (client != null && client.isConnected()) {
                    client.disconnectForcibly(); // 强制断开旧连接
                    client.close(true);
                }
                // 重新初始化客户端
                client = new MqttAsyncClient(brokerUrl, clientId, new MemoryPersistence());
                client.setCallback(this); // 关键：重新绑定回调
                client.reconnect();
                resubscribeAllTopics(); // 重连后立即订阅
                log.info("重连成功");
            } catch (MqttException e) {
                log.error("重连失败", e);
                scheduleReconnect(attempt + 1);
            }
        }, delay, TimeUnit.MILLISECONDS);
    }

    /**
     * 发布消息
     * @param topic 主题
     * @param payload 消息体
     * @param qos QoS等级
     */
    @Async
    public void publish(String topic, String payload, int qos) {
        try {
            if (!client.isConnected()) {
                throw new MqttException(MqttException.REASON_CODE_CLIENT_NOT_CONNECTED);
            }
            client.publish(topic, payload.getBytes(), qos, false);
        } catch (MqttException e) {
            log.error("消息发送失败 [topic: {}]", topic, e);
        }
    }

    @Async
    public void publish(String topic, String payload) {
        try {
            if (!client.isConnected()) {
                throw new MqttException(MqttException.REASON_CODE_CLIENT_NOT_CONNECTED);
            }
            client.publish(topic, payload.getBytes(), 1, false);
        } catch (MqttException e) {
            log.error("消息发送失败 [topic: {}]", topic, e);
        }
    }

    /**
     * 订阅主题
     * @param topicFilter 主题过滤器
     * @param qos QoS等级
     * @param handler 消息处理器
     */
    public void subscribe(String topicFilter, int qos, IMessageHandler handler) {
        try {
            if (!client.isConnected()) {
                log.warn("客户端未连接，延迟订阅 [topic: {}]", topicFilter);
                return;
            }
            client.subscribe(topicFilter, qos);
            topicHandlers.computeIfAbsent(topicFilter, k -> new ArrayList<>())
                    .add(handler);
            log.info("订阅成功: {}", topicFilter);
        } catch (MqttException e) {
            log.error("订阅失败 [topic: {}]", topicFilter);
            e.printStackTrace();
        }
    }

    private void resubscribeAllTopics() {
        topicHandlers.forEach((topic, handlers) -> {
            int retry = 0;
            while (retry < 3) {
                try {
                    client.subscribe(topic, 1);
                    log.info("主题重新订阅成功: {}", topic);
                    break;
                } catch (MqttException e) {
                    retry++;
                    log.error("重新订阅失败 [topic: {}], 重试 {}/3", topic, retry);
                    sleep(1000L * retry);
                }
            }
        });
    }

    /**
     * 消息到达处理
     * @param topic 主题
     * @param message 消息
     */
    @Override
    public void messageArrived(String topic, MqttMessage message) {

        topicHandlers.entrySet().stream()
                .filter(entry -> MqttTopic.isMatched(entry.getKey(), topic))
                .flatMap(entry -> entry.getValue().stream())
                .forEach(handler -> {
                    try {
                        handler.handleMessage(topic, message);
                    } catch (Exception e) {
                        log.error("消息处理异常 [handler: {}]", handler.getClass(), e);
                    }
                });
    }

    @Override
    public void connectionLost(Throwable cause) {
        log.error("连接丢失", cause);
        scheduleReconnect(1);
    }

    @Override
    public void deliveryComplete(IMqttDeliveryToken token) {
        // QoS处理逻辑
    }

    @Override
    public void connectComplete(boolean reconnect, String serverURI) {
        log.info("连接已建立 [reconnect: {}]", reconnect);
        if (reconnect) {
            resubscribeAllTopics();
            autoRegisterHandlers();
        }
    }

    /**
     * 自动注册处理器
     */
    private void autoRegisterHandlers() {
        context.getBeansOfType(IMessageHandler.class).values()
                .forEach(handler -> {
                    handler.getTopics().forEach(topic ->
                            subscribe(topic.getTopicFilter(), topic.getQos(), handler));
                });
    }
}